@@ -9,7 +9,12 @@
 
  test/aaip_0_2.h - Public declarations
 
- Copyright (c) 2009 - 2016 Thomas Schmitt, libburnia project, GPLv2+
+ Copyright (c) 2009 - 2016 Thomas Schmitt
+
+ This file is part of the libisofs project; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License version 2
+ or later as published by the Free Software Foundation.
+ See COPYING file for details.
 
 */
 
@@ -507,7 +512,8 @@ int aaip_set_acl_text(char *path, char *text, int flag);
     ISO_AAIP_ACL_MULT_OBJ multiple entries of user::, group::, other::
 */
 int aaip_set_attr_list(char *path, size_t num_attrs, char **names,
-                       size_t *value_lengths, char **values, int flag);
+                       size_t *value_lengths, char **values,
+                       int *errnos, int flag);
 
 #endif /* ! Aaip_h_is_includeD */
 
