@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2009 - 2017 Thomas Schmitt
+ * Copyright (c) 2009 - 2019 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -476,12 +476,19 @@ struct iso_write_opts {
     char *efi_boot_partition;
     int efi_boot_part_flag;
 
-    /* Eventual disk file paths of prepared images which shall be appended
-       after the ISO image and described by partiton table entries in a MBR
+    /* Disk file paths of prepared images which shall be appended
+       after the ISO image and described by partiton table entries in a MBR.
+       NULL means unused.
     */
     char *appended_partitions[ISO_MAX_PARTITIONS]; 
     uint8_t appended_part_types[ISO_MAX_PARTITIONS];
     int appended_part_flags[ISO_MAX_PARTITIONS];
+    uint8_t appended_part_type_guids[ISO_MAX_PARTITIONS][16];
+
+    /* Flags in case that appended partitions show up in GPT:
+       bit0= appended_part_type_guids is valid
+    */
+    uint8_t appended_part_gpt_flags[ISO_MAX_PARTITIONS];
 
     /* If 1: With appended partitions: create protective MBR and mark by GPT
      */
@@ -504,6 +511,14 @@ struct iso_write_opts {
     */
     int iso_mbr_part_type;
 
+    /* iso_write_opts_set_iso_type_guid
+     */
+    uint8_t iso_gpt_type_guid[16];
+    /* bit0= iso_gpt_type_guid is valid
+    */
+    int iso_gpt_flag;
+
+
     /* Eventual name of the non-ISO aspect of the image. E.g. SUN ASCII label.
      */
     char ascii_disc_label[ISO_DISC_LABEL_SIZE];
@@ -799,6 +814,11 @@ struct ecma119_image
     */
     IsoFileSrc *sparc_core_src;
 
+    /* Trailing padding of ISO filesystem partition for cylinder alignment */
+    /* Only in effect with Libisofs_part_align_writeR */
+    uint32_t part_align_blocks;
+    uint32_t alignment_end_block;
+
     /* Counted in blocks of 2048 */
     uint32_t appended_part_prepad[ISO_MAX_PARTITIONS];
     uint32_t appended_part_start[ISO_MAX_PARTITIONS];
@@ -1044,6 +1064,17 @@ int iso_interval_reader_start_size(Ecma119Image *t, char *path,
                                    off_t *start_byte, off_t *byte_count,
                                    int flag);
 
+/* Obtains start and end number of appended partition range and returns
+   the number of valid entries in the list of appended partitions.
+*/
+int iso_count_appended_partitions(Ecma119Image *target,
+                                  int *first_partition, int *last_partition);
 
+/* Determines the range of valid partition numbers depending on partition
+   table type.
+*/
+void iso_tell_max_part_range(IsoWriteOpts *opts,
+                             int *first_partition, int *last_partition,
+                             int flag);
 
 #endif /*LIBISO_ECMA119_H_*/
