@@ -130,9 +130,9 @@ int ziso_running_new(ZisofsFilterRuntime **running, int flag)
 
     o->block_size = ziso_block_size;
 #ifdef Libisofs_with_zliB
-    o->buffer_size= compressBound((uLong) ziso_block_size);
+    o->buffer_size = compressBound((uLong) ziso_block_size);
 #else
-    o->buffer_size= 2 * ziso_block_size;
+    o->buffer_size = 2 * ziso_block_size;
 #endif
     o->read_buffer = calloc(o->block_size, 1);
     o->block_buffer = calloc(o->buffer_size, 1);
@@ -381,7 +381,7 @@ int ziso_stream_compress(IsoStream *stream, void *buf, size_t desired)
                     if (todo * 4 > rng->buffer_size)
                         todo = rng->buffer_size / 4;
                     memcpy(rng->block_buffer,
-                           data->block_pointers + 4 * rng->block_pointer_rpos,
+                           data->block_pointers + rng->block_pointer_rpos,
                            todo * 4);
                     rng->buffer_rpos = 0;
                     rng->buffer_fill = todo * 4;
@@ -838,6 +838,9 @@ no_mem:
 static
 int ziso_cmp_ino(IsoStream *s1, IsoStream *s2);
 
+static
+int ziso_uncompress_cmp_ino(IsoStream *s1, IsoStream *s2);
+
 
 IsoStreamIface ziso_stream_compress_class = {
     4,
@@ -868,7 +871,7 @@ IsoStreamIface ziso_stream_uncompress_class = {
     ziso_stream_free,
     ziso_update_size,
     ziso_get_input_stream,
-    ziso_cmp_ino,
+    ziso_uncompress_cmp_ino,
     ziso_clone_stream
 };
 
@@ -876,9 +879,36 @@ IsoStreamIface ziso_stream_uncompress_class = {
 static
 int ziso_cmp_ino(IsoStream *s1, IsoStream *s2)
 {
+    /* This function may rely on being called by iso_stream_cmp_ino()
+       only with s1, s2 which both point to it as their .cmp_ino() function. 
+       It would be a programming error to let any other than
+       ziso_stream_compress_class point to ziso_cmp_ino().
+    */
     if (s1->class != s2->class || (s1->class != &ziso_stream_compress_class &&
                                    s2->class != &ziso_stream_uncompress_class))
         iso_stream_cmp_ino(s1, s2, 1);
+
+    /* Both streams apply the same treatment to their input streams */
+    return iso_stream_cmp_ino(iso_stream_get_input_stream(s1, 0),
+                              iso_stream_get_input_stream(s2, 0), 0);
+}
+
+
+static
+int ziso_uncompress_cmp_ino(IsoStream *s1, IsoStream *s2)
+{
+    /* This function may rely on being called by iso_stream_cmp_ino()
+       only with s1, s2 which both point to it as their .cmp_ino() function. 
+       It would be a programming error to let any other than
+       ziso_stream_uncompress_class point to ziso_uncompress_cmp_ino().
+       This fallback endangers transitivity of iso_stream_cmp_ino().
+    */
+    if (s1->class != s2->class ||
+        (s1->class != &ziso_stream_uncompress_class &&
+         s2->class != &ziso_stream_uncompress_class))
+        iso_stream_cmp_ino(s1, s2, 1);
+
+    /* Both streams apply the same treatment to their input streams */
     return iso_stream_cmp_ino(iso_stream_get_input_stream(s1, 0),
                               iso_stream_get_input_stream(s2, 0), 0);
 }
