@@ -203,6 +203,7 @@ int iso_image_new(const char *name, IsoImage **image)
         img->hfsplus_blessed[i] = NULL;
     img->collision_warnings = 0;
     img->imported_sa_info = NULL;
+    img->blind_on_local_get_attrs = 0;
 
     *image = img;
     return ISO_SUCCESS;
@@ -610,6 +611,15 @@ void iso_image_set_ignore_aclea(IsoImage *image, int what)
 {
     image->builder_ignore_acl = (what & 1);
     image->builder_ignore_ea = !!(what & 2);
+    image->builder_take_all_ea = !!(what & 8);
+}
+
+
+int iso_image_get_ignore_aclea(IsoImage *image)
+{
+    return image->builder_ignore_acl |
+           (image->builder_ignore_ea << 1) |
+           (image->builder_take_all_ea << 3);
 }
 
 
@@ -1128,3 +1138,15 @@ int iso_image_truncate_name(IsoImage *image, const char *name, char **namept,
     return ret;
 }
 
+
+/* API */
+int iso_image_was_blind_attrs(IsoImage *image, int flag)
+{
+    int ret;
+
+    ret = image->blind_on_local_get_attrs;
+    if (flag & 1)
+        image->blind_on_local_get_attrs = 0;
+    return ret;
+}
+
