@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
  * Copyright (c) 2007 Mario Danic
- * Copyright (c) 2011-2012 Thomas Schmitt
+ * Copyright (c) 2011-2018 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -28,31 +28,69 @@
 #include <stdio.h>
 #include <string.h>
 
-static
-int get_joliet_name(Ecma119Image *t, IsoNode *iso, uint16_t **name)
+/* @param flag   bit0=  Do not issue error messages
+*/
+int iso_get_joliet_name(IsoWriteOpts *opts, char *input_charset, int imgid,
+                        char *node_name, enum IsoNodeType node_type,
+                        size_t *joliet_ucs2_failures,
+                        uint16_t **name, int flag)
 {
-    int ret;
-    uint16_t *ucs_name;
+    int ret = ISO_SUCCESS;
+    uint16_t *ucs_name = NULL, *utf16_name = NULL;
     uint16_t *jname = NULL;
 
-    if (iso->name == NULL) {
+    if (node_name == NULL) {
         /* it is not necessarily an error, it can be the root */
+        *name = NULL;
         return ISO_SUCCESS;
     }
 
-    ret = str2ucs(t->input_charset, iso->name, &ucs_name);
-    if (ret < 0) {
-        iso_msg_debug(t->image->id, "Can't convert %s", iso->name);
-        return ret;
+    if (opts->joliet_utf16) {
+        ret = str2utf16be(input_charset, node_name, &ucs_name);
+        if (ret < 0) {
+            if (!(flag & 512))
+                iso_msg_debug(imgid, "Cannot convert to UTF-16 : \"%s\"",
+                              node_name);
+            goto ex;
+        }
+    } else {
+        ret = str2ucs(input_charset, node_name, &ucs_name);
+        if (ret < 0) {
+            if (!(flag & 512))
+                iso_msg_debug(imgid, "Cannot convert to UCS-2 : \"%s\"",
+                              node_name);
+            goto ex;
+        }
+        ret = str2utf16be(input_charset, node_name, &utf16_name);
+        if (ret == ISO_SUCCESS) {
+            if (ucscmp(ucs_name, utf16_name) != 0) {
+                (*joliet_ucs2_failures)++;
+                if (*joliet_ucs2_failures <= ISO_JOLIET_UCS2_WARN_MAX &&
+                    !(flag & 512)) {
+                    iso_msg_submit(imgid, ISO_NAME_NOT_UCS2, 0,
+               "Filename not suitable for Joliet character set UCS-2 : \"%s\"",
+                                   node_name);
+                }
+            }
+        }
     }
-    if (iso->type == LIBISO_DIR) {
-        jname = iso_j_dir_id(ucs_name, t->joliet_long_names << 1);
+    if (node_type == LIBISO_DIR) {
+        jname = iso_j_dir_id(ucs_name, opts->joliet_long_names << 1);
     } else {
         jname = iso_j_file_id(ucs_name,
-                       (t->joliet_long_names << 1) | !!(t->no_force_dots & 2));
+                 (opts->joliet_long_names << 1) | !!(opts->no_force_dots & 2));
     }
-    free(ucs_name);
-    if (jname != NULL) {
+    ret = ISO_SUCCESS;
+ex:;
+    if (ucs_name != NULL)
+        free(ucs_name);
+    if (utf16_name != NULL)
+        free(utf16_name);
+    if (ret != ISO_SUCCESS) {
+        if (jname != NULL)
+            free(jname);
+        return ret;
+    } else if (jname != NULL) {
         *name = jname;
         return ISO_SUCCESS;
     } else {
@@ -64,6 +102,19 @@ int get_joliet_name(Ecma119Image *t, IsoNode *iso, uint16_t **name)
     }
 }
 
+static
+int get_joliet_name(Ecma119Image *t, IsoNode *iso, uint16_t **name)
+{
+    int ret;
+
+    ret = iso_get_joliet_name(t->opts, t->input_charset, t->image->id,
+                              iso->name, iso->type, &(t->joliet_ucs2_failures),
+                              name, 0);
+
+    return ret;
+}
+
+
 static
 void joliet_node_free(JolietNode *node)
 {
@@ -75,7 +126,8 @@ void joliet_node_free(JolietNode *node)
         for (i = 0; i < node->info.dir->nchildren; i++) {
             joliet_node_free(node->info.dir->children[i]);
         }
-        free(node->info.dir->children);
+        if (node->info.dir->children != NULL)
+            free(node->info.dir->children);
         free(node->info.dir);
     }
     iso_node_unref(node->node);
@@ -106,11 +158,14 @@ int create_node(Ecma119Image *t, IsoNode *iso, JolietNode **node)
             free(joliet);
             return ISO_OUT_OF_MEM;
         }
-        joliet->info.dir->children = calloc(sizeof(void*), dir->nchildren);
-        if (joliet->info.dir->children == NULL) {
-            free(joliet->info.dir);
-            free(joliet);
-            return ISO_OUT_OF_MEM;
+        joliet->info.dir->children = NULL;
+        if (dir->nchildren > 0) {
+            joliet->info.dir->children = calloc(sizeof(void*), dir->nchildren);
+            if (joliet->info.dir->children == NULL) {
+                free(joliet->info.dir);
+                free(joliet);
+                return ISO_OUT_OF_MEM;
+            }
         }
         joliet->type = JOLIET_DIR;
     } else if (iso->type == LIBISO_FILE) {
@@ -120,7 +175,8 @@ int create_node(Ecma119Image *t, IsoNode *iso, JolietNode **node)
         IsoFile *file = (IsoFile*) iso;
 
         size = iso_stream_get_size(file->stream);
-        if (size > (off_t)MAX_ISO_FILE_SECTION_SIZE && t->iso_level != 3) {
+        if (size > (off_t)MAX_ISO_FILE_SECTION_SIZE &&
+            t->opts->iso_level != 3) {
             char *ipath = iso_tree_get_node_path(iso);
             free(joliet);
             ret = iso_msg_submit(t->image->id, ISO_FILE_TOO_BIG, 0,
@@ -188,7 +244,7 @@ int create_tree(Ecma119Image *t, IsoNode *iso, JolietNode **tree, int pathlen)
         return ret;
     }
     max_path = pathlen + 1 + (jname ? ucslen(jname) * 2 : 0);
-    if (!t->joliet_longer_paths && max_path > 240) {
+    if (!t->opts->joliet_longer_paths && max_path > 240) {
         char *ipath = iso_tree_get_node_path(iso);
         /*
          * Wow!! Joliet is even more restrictive than plain ISO-9660,
@@ -249,7 +305,7 @@ int create_tree(Ecma119Image *t, IsoNode *iso, JolietNode **tree, int pathlen)
         {
             char *ipath = iso_tree_get_node_path(iso);
             ret = iso_msg_submit(t->image->id, ISO_FILE_IGNORED, 0,
-                 "Can't add %s to Joliet tree. %s can only be added to a "
+                 "Cannot add %s to Joliet tree. %s can only be added to a "
                  "Rock Ridge tree.", ipath, (iso->type == LIBISO_SYMLINK ?
                                              "Symlinks" : "Special files"));
             free(ipath);
@@ -281,6 +337,8 @@ void sort_tree(JolietNode *root)
 {
     size_t i;
 
+    if (root->info.dir->children == NULL)
+        return;
     qsort(root->info.dir->children, root->info.dir->nchildren,
           sizeof(void*), cmp_node);
     for (i = 0; i < root->info.dir->nchildren; i++) {
@@ -343,6 +401,20 @@ int joliet_create_mangled_name(uint16_t *dest, uint16_t *src, int digits,
     return ISO_SUCCESS;
 }
 
+/*
+ * From Joliet specs:
+ * "ISO 9660 (Section 7.5.1) states that the sum of the following shall not
+ *  exceed 30:
+ *  - If there is a file name, the length of the file name.
+ *  - If there is a file name extension, the length of the file name extension.
+ *  On Joliet compliant media, however, the sum as calculated above shall not
+ *  exceed 128 [bytes], to allow for longer file identifiers."
+ *
+ * I.e. the dot does not count.
+ *
+ * (We have an option to lift the limit from 64*2 to 103*2, which is the
+ *  maximum to fit into an ISO 9660 directory record.)
+ */
 static
 int mangle_single_dir(Ecma119Image *t, JolietNode *dir)
 {
@@ -354,12 +426,16 @@ int mangle_single_dir(Ecma119Image *t, JolietNode *dir)
     uint16_t *full_name = NULL;
     uint16_t *tmp = NULL;
 
+    nchildren = dir->info.dir->nchildren;
+    if (nchildren <= 0) {
+        ret = ISO_SUCCESS;
+        goto ex;
+    }
+    children = dir->info.dir->children;
     LIBISO_ALLOC_MEM(full_name, uint16_t, LIBISO_JOLIET_NAME_MAX);
     LIBISO_ALLOC_MEM(tmp, uint16_t, LIBISO_JOLIET_NAME_MAX);
-    nchildren = dir->info.dir->nchildren;
-    children = dir->info.dir->children;
 
-    if (t->joliet_long_names)
+    if (t->opts->joliet_long_names)
         maxchar = 103;
 
     /* a hash table will temporary hold the names, for fast searching */
@@ -419,25 +495,26 @@ int mangle_single_dir(Ecma119Image *t, JolietNode *dir)
                 ext = dot + 1;
 
                 extlen = ucslen(ext);
-                max = maxchar + 1 - extlen - 1 - digits;
+                max = maxchar - extlen - digits;
                 if (max <= 0) {
-                    /* this can happen if extension is too long */
-                    if (extlen + max > 3) {
+                    /*
+                     * This can happen if the extension is too long.
+                     * Reduce its length, to give name at least one
+                     * original character, if it has any.
+                     */
+                    max = (dot > full_name);
+                    extlen = maxchar - max - digits;
+                    if (extlen < 3) {
                         /*
-                         * reduce extension len, to give name an extra char
-                         * note that max is negative or 0
-                         */
-                        extlen = extlen + max - 1;
-                        ext[extlen] = 0;
-                        max = maxchar + 2 - extlen - 1 - digits;
-                    } else {
-                        /*
-                         * error, we don't support extensions < 3
-                         * This can't happen with current limit of digits.
+                         * error, we do not reduce extensions to length < 3
+                         *
+                         * This cannot happen with current limit of digits
+                         * because maxchar is at least 64 and digits at most 7.
                          */
                         ret = ISO_ERROR;
                         goto mangle_cleanup;
                     }
+                    ext[extlen] = 0;
                 }
                 /* ok, reduce name by digits */
                 if (name + max < dot) {
@@ -446,10 +523,10 @@ int mangle_single_dir(Ecma119Image *t, JolietNode *dir)
             } else {
                 /* Directory, or file without extension */
                 if (children[i]->type == JOLIET_DIR) {
-                    max = maxchar + 1 - digits;
+                    max = maxchar - digits;
                     dot = NULL; /* dots have no meaning in dirs */
                 } else {
-                    max = maxchar + 1 - digits;
+                    max = maxchar - digits;
                 }
                 name = full_name;
                 if ((size_t) max < ucslen(name)) {
@@ -598,7 +675,7 @@ size_t calc_dirent_len(Ecma119Image *t, JolietNode *n)
 {
     /* note than name len is always even, so we always need the pad byte */
     int ret = n->name ? ucslen(n->name) * 2 + 34 : 34;
-    if (n->type == JOLIET_FILE && !(t->omit_version_numbers & 3)) {
+    if (n->type == JOLIET_FILE && !(t->opts->omit_version_numbers & 3)) {
         /* take into account version numbers */
         ret += 4;
     }
@@ -717,7 +794,7 @@ int joliet_writer_compute_data_blocks(IsoImageWriter *writer)
     t->curblock += DIV_UP(path_table_size, BLOCK_SIZE);
     t->joliet_path_table_size = path_table_size;
 
-    if (t->partition_offset > 0) {
+    if (t->opts->partition_offset > 0) {
         /* Take into respect second directory tree */
         ndirs = t->joliet_ndirs;
         t->joliet_ndirs = 0;
@@ -766,7 +843,7 @@ void write_one_dir_record(Ecma119Image *t, JolietNode *node, int file_id,
 
     memcpy(rec->file_id, name, len_fi);
 
-    if (node->type == JOLIET_FILE && !(t->omit_version_numbers & 3)) {
+    if (node->type == JOLIET_FILE && !(t->opts->omit_version_numbers & 3)) {
         len_dr += 4;
         rec->file_id[len_fi++] = 0;
         rec->file_id[len_fi++] = ';';
@@ -801,13 +878,13 @@ void write_one_dir_record(Ecma119Image *t, JolietNode *node, int file_id,
     iso_bb(rec->block, block - t->eff_partition_offset, 4);
     iso_bb(rec->length, len, 4);
 
-    /* was: iso_datetime_7(rec->recording_time, t->now, t->always_gmt);
+    /* was: iso_datetime_7(rec->recording_time, t->now, t->opts->always_gmt);
     */
     iso= node->node;
     iso_datetime_7(rec->recording_time, 
-                   (t->dir_rec_mtime & 2) ? ( t->replace_timestamps ?
-                                              t->timestamp : iso->mtime )
-                                          : t->now, t->always_gmt);
+                   (t->opts->dir_rec_mtime & 2) ? ( t->replace_timestamps ?
+                                                    t->timestamp : iso->mtime )
+                                                : t->now, t->opts->always_gmt);
 
     rec->flags[0] = ((node->type == JOLIET_DIR) ? 2 : 0) | (multi_extend ? 0x80 : 0);
     iso_bb(rec->vol_seq_number, (uint32_t) 1, 2);
@@ -828,18 +905,22 @@ void ucsncpy_pad(uint16_t *dest, const uint16_t *src, size_t max)
     csrc = (char*)src;
 
     if (src != NULL) {
-        len = MIN(ucslen(src) * 2, max);
+        len = MIN(ucslen(src) * 2, max - (max % 2));
     } else {
         len = 0;
     }
 
     for (i = 0; i < len; ++i)
         cdest[i] = csrc[i];
+    if (len >= 2)
+        iso_handle_split_utf16(dest + (len / 2 - 1));
 
-    for (i = len; i < max; i += 2) {
+    for (i = len; i + 1 < max; i += 2) {
         cdest[i] = '\0';
         cdest[i + 1] = ' ';
     }
+    if (max % 2)
+        cdest[max - 1] = 0;
 }
 
 int joliet_writer_write_vol_desc(IsoImageWriter *writer)
@@ -958,7 +1039,8 @@ int write_one_dir(Ecma119Image *t, JolietNode *dir)
         /* compute len of directory entry */
         fi_len = ucslen(child->name) * 2;
         len = fi_len + 34;
-        if (child->type == JOLIET_FILE && !(t->omit_version_numbers & 3)) {
+        if (child->type == JOLIET_FILE &&
+            !(t->opts->omit_version_numbers & 3)) {
             len += 4;
         }
 
@@ -1160,8 +1242,8 @@ int joliet_writer_write_data(IsoImageWriter *writer)
     if (ret < 0)
         return ret;
 
-    if (t->partition_offset > 0) {
-        t->eff_partition_offset = t->partition_offset;
+    if (t->opts->partition_offset > 0) {
+        t->eff_partition_offset = t->opts->partition_offset;
         ret = joliet_writer_write_dirs(writer);
         t->eff_partition_offset = 0;
         if (ret < 0)
@@ -1209,9 +1291,9 @@ int joliet_writer_create(Ecma119Image *target)
     /* add this writer to image */
     target->writers[target->nwriters++] = writer;
 
-    if(target->partition_offset > 0) {
+    if(target->opts->partition_offset > 0) {
         /* Create second tree */
-        target->eff_partition_offset = target->partition_offset;
+        target->eff_partition_offset = target->opts->partition_offset;
         ret = joliet_tree_create(target);
         if (ret < 0) {
             return ret;
