@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2009 - 2016 Thomas Schmitt
+ * Copyright (c) 2009 - 2019 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -1365,7 +1365,11 @@ int iso_dir_insert(IsoDir *dir, IsoNode *node, IsoNode **pos,
                 /* old file is newer */
                 return ISO_NODE_NAME_NOT_UNIQUE;
             }
-            /* fall down */
+            if ((node->mode & S_IFMT) != ((*pos)->mode & S_IFMT)) {
+                /* different file types */
+                return ISO_NODE_NAME_NOT_UNIQUE;
+            }
+            break;
         case ISO_REPLACE_IF_SAME_TYPE:
             if ((node->mode & S_IFMT) != ((*pos)->mode & S_IFMT)) {
                 /* different file types */
@@ -1456,6 +1460,7 @@ void iso_notify_dir_iters(IsoNode *node, int flag)
 int iso_node_new_root(IsoDir **root)
 {
     IsoDir *dir;
+    time_t now;
 
     dir = calloc(1, sizeof(IsoDir));
     if (dir == NULL) {
@@ -1463,7 +1468,8 @@ int iso_node_new_root(IsoDir **root)
     }
     dir->node.refcount = 1;
     dir->node.type = LIBISO_DIR;
-    dir->node.atime = dir->node.ctime = dir->node.mtime = time(NULL);
+    iso_nowtime(&now, 0);
+    dir->node.atime = dir->node.ctime = dir->node.mtime = now;
     dir->node.mode = S_IFDIR | 0555;
 
     /* set parent to itself, to prevent root to be added to another dir */
@@ -1824,6 +1830,7 @@ int attr_enlarge_list(char ***names, size_t **value_lengths, char ***values,
                               (but not if bit2 is set)
                         bit2= delete the given names rather than overwrite
                               their content
+                        bit3= with bit0: delete all old non-"isofs." names
                         bit4= do not overwrite value of empty name
                         bit5= do not overwrite isofs attributes 
                         bit15= release memory and return 1
@@ -1849,9 +1856,11 @@ int iso_node_merge_xattr(IsoNode *node, size_t num_attrs, char **names,
         return ret;
 
     if ((flag & 1) && (!(flag & 4))) {
-        /* Delete unmatched user space pairs */
+        /* Delete unmatched settable pairs */
         for (j = 0; j < *m_num_attrs; j++) {
-            if (strncmp((*m_names)[j], "user.", 5) != 0)
+            if (strncmp((*m_names)[j], "isofs.", 6) == 0)
+                continue;
+            if (strncmp((*m_names)[j], "user.", 5) != 0 && !(flag & 8))
                 continue;
             for (i = 0; i < num_attrs; i++) {
                 if (names[i] == NULL || (*m_names)[j] == NULL)
@@ -1980,7 +1989,7 @@ int iso_node_set_attrs(IsoNode *node, size_t num_attrs, char **names,
                   node, num_attrs, names, value_lengths, values,
                   &m_num, &m_names, &m_value_lengths, &m_values,
                   (flag & 4) | (!(flag & 2)) | ((!(flag & 1)) << 4) |
-                  ((flag & 16) << 1));
+                  ((flag & 16) << 1) | (flag & 8));
         if (ret < 0)
             goto ex;
         num_attrs = m_num;
@@ -2252,12 +2261,14 @@ int iso_node_set_acl_text(IsoNode *node, char *access_text, char *default_text,
                 goto ex;
             }
             ret = aaip_encode_both_acl(a_text, d_text, st_mode,
-                                       &acl_len, &acl, 2 | 8);
+                                       &acl_len, &acl,
+                                       2 | 8 | ((flag & 4) << 2));
         } else {
             ret = 1;
             if (access_text != NULL || default_text != NULL)
                 ret = aaip_encode_both_acl(access_text, default_text, st_mode,
-                                           &acl_len, &acl, 2 | 8);
+                                           &acl_len, &acl,
+                                           2 | 8 | ((flag & 4) << 2));
         }
         if (ret == -1)
             ret = ISO_OUT_OF_MEM;
@@ -2316,7 +2327,8 @@ int iso_node_set_acl_text(IsoNode *node, char *access_text, char *default_text,
         goto ex;
     }
     ret = aaip_encode_both_acl(access_text, default_text,
-                               st_mode, &acl_len, &acl, 2 | 8);
+                               st_mode, &acl_len, &acl,
+                               2 | 8 | ((flag & 4) << 2));
     if (ret < -3)
         goto ex;
     if (ret <= 0) {
