@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
  * Copyright (c) 2007 Mario Danic
- * Copyright (c) 2009 - 2015 Thomas Schmitt
+ * Copyright (c) 2009 - 2019 Thomas Schmitt
  * 
  * This file is part of the libisofs project; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License version 2 
@@ -18,7 +18,6 @@
 #include "messages.h"
 #include "joliet.h"
 #include "node.h"
-#include "../version.h"
 
 #include <stdlib.h>
 #include <wchar.h>
@@ -1823,7 +1822,7 @@ time_t iso_datetime_read_17(const uint8_t *buf)
     tm.tm_mon -= 1;
     tm.tm_isdst = 0;
 
-    return timegm(&tm) - ((int8_t)buf[6]) * 60 * 15;
+    return timegm(&tm) - ((int8_t)buf[16]) * 60 * 15;
 }
 
 /**
@@ -1978,9 +1977,18 @@ ex:;
 
 void iso_lib_version(int *major, int *minor, int *micro)
 {
+
+    *major = iso_lib_header_version_major;
+    *minor = iso_lib_header_version_minor;
+    *micro = iso_lib_header_version_micro;
+
+/* No more: values from version.h generated from version.h.in and
+            macro values defined in configure.ac
+
     *major = LIBISOFS_MAJOR_VERSION;
     *minor = LIBISOFS_MINOR_VERSION;
     *micro = LIBISOFS_MICRO_VERSION;
+*/
 }
 
 int iso_lib_is_compatible(int major, int minor, int micro)
@@ -2448,3 +2456,27 @@ int iso_truncate_leaf_name(int mode, int length, char *name, int flag)
     return ret;
 }
 
+/* API */
+/* @param flag bit0= *now contains the time to be set as nowtime override
+               bit1= disable the nowtime override
+   @return 1= *now is not overridden , 2= *now is overridden
+*/
+int iso_nowtime(time_t *now, int flag)
+{
+    static int now_time_overridden = 0;
+    static time_t now_time_override = 0;
+
+    if (flag & 1) {
+        now_time_overridden = 1;
+        now_time_override = *now;
+    }
+    if (flag & 2) {
+        now_time_overridden = 0;
+    }
+    *now = time(NULL);
+    if (!now_time_overridden)
+        return 1;
+    *now = now_time_override;
+    return 2;
+}
+
