@@ -21,6 +21,12 @@
 
 #include <time.h>
 
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+
+#include <fcntl.h>
+
 #ifndef MAX
 #   define MAX(a, b) (((a) > (b)) ? (a) : (b))
 #endif
@@ -56,9 +62,16 @@ int iso_init_locale(int flag);
 int strconv(const char *input, const char *icharset, const char *ocharset,
             char **output);
 
+/* Like strconv but processing len input bytes rather than strlen(input)
+ */
 int strnconv(const char *str, const char *icharset, const char *ocharset,
              size_t len, char **output);
 
+/* Like strnconv but also returning the number of bytes in *output.
+ */
+int strnconvl(const char *str, const char *icharset, const char *ocharset,
+              size_t len, char **output, size_t *out_len);
+
 /**
  * Convert a given string from any input charset to ASCII
  * 
@@ -236,6 +249,12 @@ uint16_t *ucscpy(uint16_t *dest, const uint16_t *src);
  */
 uint16_t *ucsncpy(uint16_t *dest, const uint16_t *src, size_t n);
 
+/**
+ * Check whether utf_word is the first surrogate word of a pair.
+ * If so, change it to UTF-16 character '_'.
+ */
+void iso_handle_split_utf16(uint16_t *utf_word);
+
 /**
  * Convert a given input string to d-chars.
  * @return
@@ -260,6 +279,9 @@ uint32_t iso_read_msb(const uint8_t *buf, int bytes);
  */
 uint32_t iso_read_bb(const uint8_t *buf, int bytes, int *error);
 
+uint64_t iso_read_lsb64(const uint8_t *buf);
+uint64_t iso_read_msb64(const uint8_t *buf);
+
 /** 
  * Records the date/time into a 7 byte buffer (ECMA-119, 9.1.5)
  * 
@@ -385,6 +407,12 @@ size_t iso_rbtree_get_size(IsoRBTree *tree);
 void **iso_rbtree_to_array(IsoRBTree *tree, int (*include_item)(void *), 
                            size_t *size);
 
+/** Predict the size of the array which gets returned by iso_rbtree_to_array().
+ */
+size_t iso_rbtree_count_array(IsoRBTree *tree, size_t initial_count,
+                              int (*include_item)(void *));
+
+
 /**
  * Create a new hash table.
  * 
@@ -531,6 +559,14 @@ int iso_util_eval_md5_tag(char *block, int desired, uint32_t lba,
 int iso_util_tag_magic(int tag_type, char **tag_magic, int *len, int flag);
 
 
+int iso_util_bin_to_hex(char *target, uint8_t *bytes, int num_bytes, int flag);
+
+int iso_util_hex_to_bin(char *hex, char *bin, int bin_size, int *bin_count,
+                        int flag);
+
+int iso_truncate_rr_name(int truncate_mode, int truncate_length,
+                         char *name, int flag);
+
 /* ------------------------------------------------------------------------- */
 
 /* In md5.h these function prototypes would be neighbors of (Ecma119Image *)
@@ -593,6 +629,29 @@ void *iso_alloc_mem(size_t size, size_t count, int flag);
         }
 
 
+/*
+   @param in    Valid memory or NULL
+   @param out   Returns valid memory or NULL
+   @param size  Number of bytes to copy. 0 means strlen(in)+1 if not NULL.
+   @return      1 or ISO_OUT_OF_MEM
+*/
+int iso_clone_mem(char *in, char **out, size_t size);
+
+/* Like iso_clone_mem but first freeing *out if not NULL
+*/
+int iso_clone_mgtd_mem(char *in, char **out, size_t size);
+
+/** Convert a text into a number of type double and multiply it by unit code
+    [kmgt] (2^10 to 2^40) or [s] (2048) or [d] (512).
+    (Also accepts capital letters.)
+    @param text Input like "42", "223062s", "3m" or "-1g"
+    @param flag Bitfield for control purposes:
+                bit0= return -1 rathern than 0 on failure
+                bit1= if scaled then compute the last byte of the last unit
+    @return The derived value
+*/
+off_t iso_scanf_io_size(char *text, int flag);
+
 /* ------------------------------------------------------------------------- */
 
 
